<li class="command">
    <div class="toggle"><p>+</p></div>
    <div class="input"><p>%s</p></div>
    <div class="duration"><p>in <b>%0.3f</b> secs</p></div>
    <ul class="validators">
        <li class='validator %s'></li>
    </ul>
    <div class="output"><p>%s</p></div>
</li>